//
// Resizable Table Columns.
//  version: 1.0
//
// (c) 2006, bz
//
// 25.12.2006:  first working prototype
// 26.12.2006:  now works in IE as well but not in Opera (Opera is @#$%!)
// 27.12.2006:  changed initialization, now just make class='resizable' in table and load script
//
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
var Resizable = {
  preventEvent: function preventEvent(e) {
    var ev = e || window.event;
    if (ev.preventDefault) {
      ev.preventDefault();
    } else {
      ev.returnValue = false;
    }
    if (ev.stopPropagation) {
      ev.stopPropagation();
    }
    return false;
  },

  getStyle: function getStyle(x, styleProp) {
    var y = undefined;
    if (x.currentStyle) {
      y = x.currentStyle[styleProp];
    } else if (window.getComputedStyle) {
      y = document.defaultView.getComputedStyle(x, null).getPropertyValue(styleProp);
    }
    return y;
  },

  getWidth: function getWidth(x) {
    var y = undefined;
    if (x.currentStyle) {
      // in IE
      y = x.clientWidth - parseInt(x.currentStyle.paddingLeft, 10) - parseInt(x.currentStyle.paddingRight, 10);
      // for IE5: let y = x.offsetWidth;
    } else if (window.getComputedStyle) {
        // in Gecko
        y = document.defaultView.getComputedStyle(x, null).getPropertyValue('width');
      }
    return y || 0;
  },

  setCookie: function setCookie(name, value, expires, path, domain, secure) {
    document.cookie = name + '=' + escape(value) + (expires ? '; expires=' + expires : '') + (path ? '; path=' + path : '') + (domain ? '; domain=' + domain : '') + (secure ? '; secure' : '');
  },

  getCookie: function getCookie(name) {
    var cookie = ' ' + document.cookie;
    var search = ' ' + name + '=';
    var setStr = null;
    var offset = 0;
    var end = 0;
    if (cookie.length > 0) {
      offset = cookie.indexOf(search);
      if (offset !== -1) {
        offset += search.length;
        end = cookie.indexOf(';', offset);
        if (end === -1) {
          end = cookie.length;
        }
        setStr = unescape(cookie.substring(offset, end));
      }
    }
    return setStr;
  },

  // main class prototype
  columnResize: function columnResize(table) {
    console.dir(table);
    if (table.tagName !== 'TABLE') return;

    this.id = table.id;

    // ============================================================
    // private data
    var self = this;

    var dragColumns = table.rows[0].cells; // first row columns, used for changing of width
    if (!dragColumns) return; // return if no table exists or no one row exists

    var dragColumnNo = undefined; // current dragging column
    var dragX = undefined; // last event X mouse coordinate

    var saveOnmouseup = undefined; // save document onmouseup event handler
    var saveOnmousemove = undefined; // save document onmousemove event handler
    var saveBodyCursor = undefined; // save body cursor property

    // ============================================================
    // methods

    // ============================================================
    // do changes columns widths
    // returns true if success and false otherwise
    this.changeColumnWidth = function (no, w) {
      if (!dragColumns) return false;

      if (no < 0) return false;
      if (dragColumns.length < no) return false;

      if (parseInt(dragColumns[no].style.width, 10) <= -w) return false;
      if (dragColumns[no + 1] && parseInt(dragColumns[no + 1].style.width, 10) <= w) return false;

      dragColumns[no].style.width = parseInt(dragColumns[no].style.width, 10) + w + 'px';
      if (dragColumns[no + 1]) {
        dragColumns[no + 1].style.width = parseInt(dragColumns[no + 1].style.width, 10) - w + 'px';
      }

      return true;
    };

    // ============================================================
    // do drag column width
    this.columnDrag = function (e) {
      e = e || window.event;
      var X = e.clientX || e.pageX;
      if (!self.changeColumnWidth(dragColumnNo, X - dragX)) {
        // stop drag!
        self.stopColumnDrag(e);
      }

      dragX = X;
      // prevent other event handling
      self.preventEvent(e);
      return false;
    };

    // ============================================================
    // stops column dragging
    this.stopColumnDrag = function (e) {
      e = e || window.event;
      if (!dragColumns) return;

      // restore handlers & cursor
      document.onmouseup = saveOnmouseup;
      document.onmousemove = saveOnmousemove;
      document.body.style.cursor = saveBodyCursor;

      // remember columns widths in cookies for server side
      var colWidth = '';
      var separator = '';
      for (var i = 0; i < dragColumns.length; i++) {
        colWidth += separator + parseInt(self.getWidth(dragColumns[i]), 10);
        separator = '+';
      }
      var expire = new Date();
      expire.setDate(expire.getDate() + 365); // year
      document.cookie = self.id + '-width=' + colWidth + '; expires=' + expire.toGMTString();

      self.preventEvent(e);
    };

    // ============================================================
    // init data and start dragging
    this.startColumnDrag = function (e) {
      e = e || window.event;

      // if not first button was clicked
      // if (e.button != 0) return;

      // remember dragging object
      dragColumnNo = (e.target || e.srcElement).parentNode.parentNode.cellIndex;
      dragX = e.clientX || e.pageX;

      // set up current columns widths in their particular attributes
      // do it in two steps to avoid jumps on page!
      var colWidth = new Array();
      for (var i = 0; i < dragColumns.length; i++) {
        colWidth[i] = parseInt(self.getWidth(dragColumns[i]), 10);
      }
      for (var i = 0; i < dragColumns.length; i++) {
        dragColumns[i].width = ''; // for sure
        dragColumns[i].style.width = colWidth[i] + 'px';
      }

      saveOnmouseup = document.onmouseup;
      document.onmouseup = self.stopColumnDrag;

      saveBodyCursor = document.body.style.cursor;
      document.body.style.cursor = 'w-resize';

      // fire!
      saveOnmousemove = document.onmousemove;
      document.onmousemove = self.columnDrag;

      self.preventEvent(e);
    };

    // prepare table header to be draggable
    // it runs during class creation
    for (var i = 0; i < dragColumns.length; i++) {
      dragColumns[i].innerHTML = '<div style="position:relative;height:100%;width:100%">' + '<div style="' + 'position:absolute;height:100%;width:5px;margin-right:-5px;' + 'left:100%;top:0px;cursor:w-resize;z-index:10;">' + '</div>' + dragColumns[i].innerHTML + '</div>';
      // BUGBUG: calculate real border width instead of 5px!!!
      dragColumns[i].firstChild.firstChild.onmousedown = this.startColumnDrag;
    }
  }

};

exports['default'] = Resizable;
module.exports = exports['default'];